@@ -119,7 +119,7 @@ def render_complex_chart(chart_type, title, x_labels=None, datasets=None, z_matr
 	"""
 	Generates a chart based on the 'Super-Schema' and returns a Base64 image string.
 	"""
-	plt.figure(figsize=(7, 5))
+	plt.figure(figsize=(10, 8))
 	plt.title(title, fontsize=12, pad=20)
 	
 	# --- 1. Basic Charts (Bar, Line, Scatter) ---
@@ -128,7 +128,7 @@ def render_complex_chart(chart_type, title, x_labels=None, datasets=None, z_matr
 		if datasets:
 			for series in datasets:
 				label = series.get("label", "Data")
-				color = series.get("color", 'black')
+				color = series.get("color", None)
 				data = series.get("data", [])
 				
 				if chart_type == "bar":
@@ -315,24 +315,11 @@ bq_tool = gtypes.Tool(
     ]
 )
 
-def sanitize_for_json(obj):
-	"""
-	Recursively converts Decimal objects to float/int so they are JSON serializable.
-	"""
-	if isinstance(obj, decimal.Decimal):
-		# Convert to int if it's a whole number, else float
-		return int(obj) if obj % 1 == 0 else float(obj)
-	elif isinstance(obj, list):
-		return [sanitize_for_json(i) for i in obj]
-	elif isinstance(obj, dict):
-		return {k: sanitize_for_json(v) for k, v in obj.items()}
-	return obj
-
 SYSTEM_PROMPT = """You are a data analyst assistant for BigQuery.
 - Prefer SELECT-only SQL.
 - When missing a column/table name, use list_datasets/list_tables/get_table_schema.
-- For final answer, provide a brief natural-language summary, include the SQL statement you created in a fenced SQL code block, and include the output from the SQL as an HTML <table> in a fenced HTML code block.
 - If user asks for a chart to illustrate the data use render_complex_chart.
+- For final answer, provide a brief natural-language summary, include the SQL statement you created in a fenced code block, and include the output from that SQL as an HTML <table> in a fenced code block.
 - The rest of what follows in this prompt are the data schema and hints on how to build the SQL queries for your data analysis.
 """
 def dispatch_tool(name: str, args: Dict[str, Any]) -> Dict[str, Any]:
@@ -346,28 +333,10 @@ def dispatch_tool(name: str, args: Dict[str, Any]) -> Dict[str, Any]:
 		if name == "run_query":
 			return json_safe(run_query(**args))
 		if name == "render_complex_chart":
-			return {'chart': render_complex_chart(**args)}
+			return {'image_base64': render_complex_chart(**args)}
 		return {"error": f"Unknown tool: {name}"}
 	except Exception as e:
 		return {"error": str(e)}
-	"""
-	try:
-		returned = {}
-		if name == "list_datasets":
-			returned = {"datasets": list_datasets(**args)}
-		elif name == "list_tables":
-			returned = {"tables": list_tables(**args)}
-		elif name == "get_table_schema":
-			returned = {"schema": get_table_schema(**args)}
-		elif name == "run_query":
-			returned = run_query(**args)
-		elif name == "render_complex_chart":
-			returned = render_complex_chart(**args)
-		else:
-			return {"error": f"Unknown tool: {name}"}
-		return sanitize_for_json(returned)
-	"""
-
 
 def chat(user_data: list[str],
 			history: list[gtypes.Content] | None = None,
@@ -405,7 +374,6 @@ def chat(user_data: list[str],
 	if history is None:
 		history = []
 
-	"""
 	if not history:
 		history.append(
 			gtypes.Content(
@@ -413,7 +381,6 @@ def chat(user_data: list[str],
 				parts=[gtypes.Part(text=f"{SYSTEM_PROMPT}\n{additional_instructions}")]
 			)
 		)
-	"""
 
 	# Add the new user message
 	history.append(
@@ -423,6 +390,7 @@ def chat(user_data: list[str],
 		)
 	)
 
+	"""
 	history.insert(
 		0,
 		gtypes.Content(
@@ -430,6 +398,7 @@ def chat(user_data: list[str],
 			parts=[gtypes.Part(text=f"{SYSTEM_PROMPT}\n{additional_instructions}")]
 		)
 	)
+	"""
 
 	# First call: model may respond with text + function_call parts
 	resp = gclient.models.generate_content(
@@ -446,36 +415,74 @@ def chat(user_data: list[str],
 
 	if not tool_calls:
 		# No tools requested → just return; caller can use resp.text safely
-		return resp
+		return [resp, history]
 
 	# Execute each requested tool
-	tool_response_contents: list[gtypes.Content] = []
-	for fc in tool_calls:
-		result = dispatch_tool(fc.name, dict(fc.args))
-		tool_response_contents.append(
-			gtypes.Content(
-				role="tool",
-				parts=[
-					gtypes.Part(
-						function_response=gtypes.FunctionResponse(
-							name=fc.name,
-							response=result,
-							id=fc.id
+
+	final_response = None
+
+	#while tool_calls:
+	if True:
+		tool_response_contents: list[gtypes.Content] = []
+		for fc in tool_calls:
+			result = dispatch_tool(fc.name, dict(fc.args))
+	
+			if fc.name == "render_complex_chart":
+				sanitized_result = json_safe(result)
+				if "image_base64" in sanitized_result:
+					print("Chart generated. Returning image to user.")
+					tool_response = {
+						"status": "Chart successfully generated and displayed to user."
+					}
+					tool_response_contents.append(
+						gtypes.Content(
+							role="tool",
+							parts=[
+								gtypes.Part(
+									function_response=gtypes.FunctionResponse(
+										name=fc.name,
+										response=tool_response,
+										id=fc.id
+									)
+								)
+							],
 						)
 					)
-				],
+					history.pop()
+					return [sanitized_result, history]
+	
+			tool_response_contents.append(
+				gtypes.Content(
+					role="tool",
+					parts=[
+						gtypes.Part(
+							function_response=gtypes.FunctionResponse(
+								name=fc.name,
+								response=result,
+								id=fc.id
+							)
+						)
+					],
+				)
 			)
+
+		# Extend history with the model's function_call turn + tool responses
+		history.append(resp.candidates[0].content)
+		history.extend(tool_response_contents)
+
+		final_resp = gclient.models.generate_content(
+			model=model,
+			contents=history,
+			config=gen_config,
 		)
 
-	# Extend history with the model's function_call turn + tool responses
-	history.append(resp.candidates[0].content)
-	history.extend(tool_response_contents)
+		#tool_calls = []
+		#for part in resp.candidates[0].content.parts:
+		#	if part.function_call:
+		#		tool_calls.append(part.function_call)
+
 
 	# Second call: model now sees tool outputs and should produce final answer text
-	final_resp = gclient.models.generate_content(
-		model=model,
-		contents=history,
-		config=gen_config,
-	)
 
-	return final_resp
+	return [final_resp, history]
+
